@@ -73,7 +73,9 @@ fn start(args: &clap::ArgMatches<'_>) -> Result<(), anyhow::Error> {
             update_central_dns(
                 runtime,
                 domain_name.clone(),
-                ips.clone(),
+                ips.iter()
+                    .map(|i| utils::parse_ip_from_cidr(i.clone()).to_string())
+                    .collect(),
                 token.clone(),
                 network_id.clone(),
             )?;
@@ -172,7 +174,7 @@ fn main() -> Result<(), anyhow::Error> {
             (@arg secret_file: -s --secret +takes_value "Path to authtoken.secret (usually detected)")
             (@arg token_file: -t --token +takes_value "Path to a file containing the ZeroTier Central token")
             (@arg wildcard: -w --wildcard "Wildcard all names in Central to point at the respective member's IP address(es)")
-            (@arg verbose: -v +multiple "Verbose logging (repeat -v for more verbosity")
+            (@arg verbose: -v +multiple "Verbose logging (repeat -v for more verbosity)")
             (@arg NETWORK_ID: +required "Network ID to query")
         )
         (@subcommand supervise =>
